/*
 * Copyright (c) 2016-2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef __PARSE_FILE_H__
#define __PARSE_FILE_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "sa_types.h"

/**
 * @fn        int sa_inputfile_get_config_info(sa_config_s *config)
 * @brief     This function to get config info
 * @param     sa_config_s		[out]  parsed config info
 * @return    sa_file_state_e	return of state
 */
	sa_error_e sa_inputfile_get_config_info(sa_config_s * config, char *file_name);

/**
 * @fn        void sa_inputfile_release_resource
 * @brief     This function to free allocated resource (memory)
 * @param     sa_config_s
 * @return    void
 */
	void sa_inputfile_release_resource(sa_config_s * config);

/**
 * @fn        void sa_split_files
 * @brief     This function to split source file into several files to check file event respectively
 * @param     source file name, 2 output files to check
 * @return    void
 */
	void sa_split_files(char *sourceFile, char *dstFile_system, char *dstFile_target);

#ifdef __cplusplus
}
#endif
#endif							/* __PARSE_FILE_H__ */
